@@ -1,13 +1,14 @@
 # -*- coding: utf-8 -*-
-
+from __future__ import print_function
 import sys, os, re
+from datetime import date
 
 # Check Sphinx version
 import sphinx
-if sphinx.__version__ < "1.0.1":
-    raise RuntimeError("Sphinx 1.0.1 or newer required")
+if sphinx.__version__ < "1.6":
+    raise RuntimeError("Sphinx 1.6 or newer required")
 
-needs_sphinx = '1.0'
+needs_sphinx = '1.6'
 
 # -----------------------------------------------------------------------------
 # General configuration
@@ -17,10 +18,11 @@ needs_sphinx = '1.0'
 # coming with Sphinx (named 'sphinx.ext.*') or your custom ones.
 
 sys.path.insert(0, os.path.abspath('../sphinxext'))
+sys.path.insert(0, os.path.abspath(os.path.dirname(__file__)))
 
-extensions = ['sphinx.ext.autodoc', 'sphinx.ext.pngmath', 'numpydoc',
+extensions = ['sphinx.ext.autodoc', 'sphinx.ext.mathjax', 'numpydoc',
               'sphinx.ext.intersphinx', 'sphinx.ext.coverage',
-              'sphinx.ext.autosummary']
+              'sphinx.ext.autosummary', 'scipyoptdoc', 'doi_role']
 
 # Determine if the matplotlib has a recent enough version of the
 # plot_directive.
@@ -50,24 +52,15 @@ master_doc = 'index'
 
 # General substitutions.
 project = 'SciPy'
-copyright = '2008-2009, The Scipy community'
+copyright = '2008-%s, The SciPy community' % date.today().year
 
 # The default replacements for |version| and |release|, also used in various
 # other places throughout the built documents.
-#
 import scipy
-# The short X.Y version (including the .devXXXX suffix if present)
-version = re.sub(r'^(\d+\.\d+)\.\d+(.*)', r'\1\2', scipy.__version__)
-if 'dev' in version:
-    # retain the .dev suffix, but clean it up
-    version = re.sub(r'(\.dev\d*).*?$', r'\1', version)
-else:
-    # strip all other suffixes
-    version = re.sub(r'^(\d+\.\d+).*?$', r'\1', version)
-# The full version, including alpha/beta/rc tags.
+version = re.sub(r'\.dev-.*$', r'.dev', scipy.__version__)
 release = scipy.__version__
 
-print "Scipy (VERSION %s) (RELEASE %s)" % (version, release)
+print("Scipy (VERSION %s)" % (version,))
 
 # There are two options for replacing |today|: either, you set today to some
 # non-false value, then it is used:
@@ -104,75 +97,58 @@ pygments_style = 'sphinx'
 # HTML output
 # -----------------------------------------------------------------------------
 
-# The style sheet to use for HTML and HTML Help pages. A file of that name
-# must exist either in Sphinx' static/ path, or in one of the custom paths
-# given in html_static_path.
-html_style = 'scipy.css'
+themedir = os.path.join(os.pardir, 'scipy-sphinx-theme', '_theme')
+if os.path.isdir(themedir):
+    html_theme = 'scipy'
+    html_theme_path = [themedir]
 
-# The name for this set of Sphinx documents.  If None, it defaults to
-# "<project> v<release> documentation".
-html_title = "%s v%s Reference Guide (DRAFT)" % (project, version)
+    if 'scipyorg' in tags:
+        # Build for the scipy.org website
+        html_theme_options = {
+            "edit_link": True,
+            "sidebar": "right",
+            "scipy_org_logo": True,
+            "rootlinks": [("https://scipy.org/", "Scipy.org"),
+                          ("https://docs.scipy.org/", "Docs")]
+        }
+    else:
+        # Default build
+        html_theme_options = {
+            "edit_link": False,
+            "sidebar": "left",
+            "scipy_org_logo": False,
+            "rootlinks": []
+        }
+        html_logo = '_static/scipyshiny_small.png'
+        html_sidebars = {'index': ['indexsidebar.html', 'searchbox.html']}
+else:
+    # Build without scipy.org sphinx theme present
+    if 'scipyorg' in tags:
+        raise RuntimeError("Get the scipy-sphinx-theme first, "
+                           "via git submodule init & update")
+    else:
+        html_style = 'scipy_fallback.css'
+        html_logo = '_static/scipyshiny_small.png'
+        html_sidebars = {'index': ['indexsidebar.html', 'searchbox.html']}
 
-# The name of an image file (within the static path) to place at the top of
-# the sidebar.
-html_logo = '_static/scipyshiny_small.png'
-
-# Add any paths that contain custom static files (such as style sheets) here,
-# relative to this directory. They are copied after the builtin static files,
-# so a file named "default.css" will overwrite the builtin "default.css".
+html_title = "%s v%s Reference Guide" % (project, version)
 html_static_path = ['_static']
-
-# If not '', a 'Last updated on:' timestamp is inserted at every page bottom,
-# using the given strftime format.
 html_last_updated_fmt = '%b %d, %Y'
 
-# Correct index page
-#html_index = "index"
-
-# If true, SmartyPants will be used to convert quotes and dashes to
-# typographically correct entities.
-#html_use_smartypants = True
-
-# Custom sidebar templates, maps document names to template names.
-html_sidebars = {
-    'index': 'indexsidebar.html'
-}
-
-# Additional templates that should be rendered to pages, maps page names to
-# template names.
 html_additional_pages = {}
-
-# If false, no module index is generated.
-html_use_modindex = True
-
-# If true, the reST sources are included in the HTML build as _sources/<name>.
+html_domain_indices = True
 html_copy_source = False
-
-# If true, an OpenSearch description file will be output, and all pages will
-# contain a <link> tag referring to it.  The value of this option must be the
-# base URL from which the finished HTML is served.
-#html_use_opensearch = ''
-
-# If nonempty, this is the file name suffix for HTML files (e.g. ".html").
 html_file_suffix = '.html'
 
-# Output file base name for HTML help builder.
 htmlhelp_basename = 'scipy'
 
-# Pngmath should try to align formulas properly
-pngmath_use_preview = True
+mathjax_path = "scipy-mathjax/MathJax.js?config=scipy-mathjax"
 
 
 # -----------------------------------------------------------------------------
 # LaTeX output
 # -----------------------------------------------------------------------------
 
-# The paper size ('letter' or 'a4').
-#latex_paper_size = 'letter'
-
-# The font size ('10pt', '11pt' or '12pt').
-#latex_font_size = '10pt'
-
 # Grouping the document tree into LaTeX files. List of tuples
 # (source start file, target name, title, author, document class [howto/manual]).
 _stdauthor = 'Written by the SciPy community'
@@ -186,58 +162,107 @@ latex_documents = [
 # the title page.
 #latex_logo = None
 
-# For "manual" documents, if this is true, then toplevel headings are parts,
-# not chapters.
-#latex_use_parts = False
-
-# Additional stuff for the LaTeX preamble.
-latex_preamble = r'''
-\usepackage{amsmath}
-
-\DeclareUnicodeCharacter{00A0}{\nobreakspace}
-
-% In the parameters etc. sections, align uniformly, and adjust label emphasis
-\usepackage{expdlist}
-\let\latexdescription=\description
-\let\endlatexdescription=\enddescription
-\renewenvironment{description}%
-{\begin{latexdescription}[\setleftmargin{60pt}\breaklabel\setlabelstyle{\bfseries\itshape}]}%
-{\end{latexdescription}}
-
-% Make Examples/etc section headers smaller and more compact
-\makeatletter
-\titleformat{\paragraph}{\normalsize\normalfont\bfseries\itshape}%
-            {\py@NormalColor}{0em}{\py@NormalColor}{\py@NormalColor}
-\titlespacing*{\paragraph}{0pt}{1ex}{0pt}
-\makeatother
-
-% Save vertical space in parameter lists and elsewhere
-\makeatletter
-\renewenvironment{quote}%
-               {\list{}{\topsep=0pt%
-                        \parsep \z@ \@plus\p@}%
-                \item\relax}%
-               {\endlist}
-\makeatother
-
-% Fix footer/header
-\renewcommand{\chaptermark}[1]{\markboth{\MakeUppercase{\thechapter.\ #1}}{}}
-\renewcommand{\sectionmark}[1]{\markright{\MakeUppercase{\thesection.\ #1}}}
-'''
-
 # Documents to append as an appendix to all manuals.
 #latex_appendices = []
 
 # If false, no module index is generated.
-latex_use_modindex = False
+latex_domain_indices = False
+
+# fix issues with Unicode characters
+latex_engine = 'xelatex'
+
+latex_elements = {
+    # The paper size ('letterpaper' or 'a4paper').
+    #
+    # 'papersize': 'letterpaper',
+
+    # The font size ('10pt', '11pt' or '12pt').
+    #
+    # 'pointsize': '10pt',
+
+    # Additional stuff for the LaTeX preamble.
+    #
+    'preamble': r'''
+% In the parameters etc. sections, align uniformly, and adjust label emphasis
+\usepackage{expdlist}
+\let\latexdescription=\description
+\let\endlatexdescription=\enddescription
+\renewenvironment{description}
+{\renewenvironment{description}
+   {\begin{latexdescription}%
+    [\setleftmargin{50pt}\breaklabel\setlabelstyle{\bfseries}]%
+   }%
+   {\end{latexdescription}}%
+ \begin{latexdescription}%
+    [\setleftmargin{15pt}\breaklabel\setlabelstyle{\bfseries\itshape}]%
+}%
+{\end{latexdescription}}
+% Fix bug in expdlist's modified \@item
+\usepackage{etoolbox}
+\makeatletter
+\patchcmd\@item{{\@breaklabel} }{{\@breaklabel}}{}{}
+% Fix bug in expdlist's way of breaking the line after long item label
+\def\breaklabel{%
+    \def\@breaklabel{%
+        \leavevmode\par
+        % now a hack because Sphinx inserts \leavevmode after term node
+        \def\leavevmode{\def\leavevmode{\unhbox\voidb@x}}%
+    }%
+}
+\makeatother
+
+% Make Examples/etc section headers smaller and more compact
+\titlespacing*{\paragraph}{0pt}{1ex}{0pt}
+
+% Save vertical space in parameter lists and elsewhere
+\makeatletter
+\renewenvironment{quote}%
+               {\list{}{\topsep=0pt\relax
+                        \parsep \z@ \@plus\p@}%
+                \item\relax}%
+               {\endlist}
+\makeatother
+% Avoid small font size in code-blocks
+\fvset{fontsize=auto}
+% Use left-alignment per default in tabulary rendered tables
+\newcolumntype{T}{L}
+% Get some useful deeper bookmarks and table of contents in PDF
+\setcounter{tocdepth}{1}
+% Fix: ≠ is unknown to XeLaTeX's default font Latin Modern
+\usepackage{newunicodechar}
+\newunicodechar{≠}{\ensuremath{\neq}}
+% Get PDF to use maximal depth bookmarks
+\hypersetup{bookmarksdepth=subparagraph}
+% reduce hyperref warnings
+\pdfstringdefDisableCommands{%
+  \let\sphinxupquote\empty
+  \let\sphinxstyleliteralintitle\empty
+  \let\sphinxstyleemphasis\empty
+}
+''',
+    # Latex figure (float) alignment
+    #
+    # 'figure_align': 'htbp',
+
+    # benefit from  Sphinx built-in workaround of LaTeX's list limitations
+    'maxlistdepth': '12',
+
+    # reduce TeX warnings about underfull boxes in the index
+    'printindex': r'\raggedright\printindex',
+
+    # avoid potential problems arising from erroneous mark-up of the
+    # \mathbf{\Gamma} type
+    'passoptionstopackages': r'\PassOptionsToPackage{no-math}{fontspec}',
+}
 
 
 # -----------------------------------------------------------------------------
 # Intersphinx configuration
 # -----------------------------------------------------------------------------
 intersphinx_mapping = {
-        'http://docs.python.org/dev': None,
-        'http://docs.scipy.org/doc/numpy': None,
+        'python': ('https://docs.python.org/dev', None),
+        'numpy': ('https://docs.scipy.org/doc/numpy', None),
+        'matplotlib': ('https://matplotlib.org', None),
 }
 
 
@@ -255,7 +280,7 @@ numpydoc_use_plots = True
 # Autosummary
 # -----------------------------------------------------------------------------
 
-if sphinx.__version__ >= "0.7": 
+if sphinx.__version__ >= "0.7":
     import glob
     autosummary_generate = glob.glob("*.rst")
 
@@ -281,23 +306,25 @@ coverage_ignore_c_items = {}
 #------------------------------------------------------------------------------
 plot_pre_code = """
 import numpy as np
-import scipy as sp
 np.random.seed(123)
 """
 plot_include_source = True
-plot_formats = [('png', 100), 'pdf']
+plot_formats = [('png', 96), 'pdf']
 plot_html_show_formats = False
+plot_html_show_source_link = False
 
 import math
 phi = (math.sqrt(5) + 1)/2
 
+font_size = 13*72/96.0  # 13 px
+
 plot_rcparams = {
-    'font.size': 8,
-    'axes.titlesize': 8,
-    'axes.labelsize': 8,
-    'xtick.labelsize': 8,
-    'ytick.labelsize': 8,
-    'legend.fontsize': 8,
+    'font.size': font_size,
+    'axes.titlesize': font_size,
+    'axes.labelsize': font_size,
+    'xtick.labelsize': font_size,
+    'ytick.labelsize': font_size,
+    'legend.fontsize': font_size,
     'figure.figsize': (3*phi, 3),
     'figure.subplot.bottom': 0.2,
     'figure.subplot.left': 0.2,
@@ -315,6 +342,7 @@ if not use_matplotlib_plot_directive:
 # Source code links
 # -----------------------------------------------------------------------------
 
+import re
 import inspect
 from os.path import relpath, dirname
 
@@ -326,7 +354,7 @@ for name in ['sphinx.ext.linkcode', 'linkcode', 'numpydoc.linkcode']:
     except ImportError:
         pass
 else:
-    print "NOTE: linkcode extension not found -- no links to source generated"
+    print("NOTE: linkcode extension not found -- no links to source generated")
 
 def linkcode_resolve(domain, info):
     """
@@ -346,36 +374,44 @@ def linkcode_resolve(domain, info):
     for part in fullname.split('.'):
         try:
             obj = getattr(obj, part)
-        except:
+        except Exception:
             return None
 
     try:
         fn = inspect.getsourcefile(obj)
-    except:
+    except Exception:
         fn = None
     if not fn:
         try:
             fn = inspect.getsourcefile(sys.modules[obj.__module__])
-        except:
+        except Exception:
             fn = None
     if not fn:
         return None
 
     try:
-        source, lineno = inspect.findsource(obj)
-    except:
+        source, lineno = inspect.getsourcelines(obj)
+    except Exception:
         lineno = None
 
     if lineno:
-        linespec = "#L%d" % (lineno + 1)
+        linespec = "#L%d-L%d" % (lineno, lineno + len(source) - 1)
     else:
         linespec = ""
 
-    fn = relpath(fn, start=dirname(scipy.__file__))
+    startdir = os.path.abspath(os.path.join(dirname(scipy.__file__), '..'))
+    fn = relpath(fn, start=startdir).replace(os.path.sep, '/')
 
-    if 'dev' in scipy.__version__:
-        return "http://github.com/scipy/scipy/blob/master/scipy/%s%s" % (
-           fn, linespec)
+    if fn.startswith('scipy/'):
+        m = re.match(r'^.*dev0\+([a-f0-9]+)$', scipy.__version__)
+        if m:
+            return "https://github.com/scipy/scipy/blob/%s/%s%s" % (
+                m.group(1), fn, linespec)
+        elif 'dev' in scipy.__version__:
+            return "https://github.com/scipy/scipy/blob/master/%s%s" % (
+                fn, linespec)
+        else:
+            return "https://github.com/scipy/scipy/blob/v%s/%s%s" % (
+                scipy.__version__, fn, linespec)
     else:
-        return "http://github.com/scipy/scipy/blob/v%s/scipy/%s%s" % (
-           scipy.__version__, fn, linespec)
+        return None
